""" Blob utils test class
"""
import unittest
from unittest import TestCase
from unittest.mock import patch

from django.conf import settings

from core_main_app.commons.exceptions import BlobDownloaderUrlParseError
from core_main_app.utils.blob_downloader import BlobDownloader
from tests.test_settings import SERVER_URI


class TestBlobDownloaderIsUrlFormLocalInstance(TestCase):
    """Test BlobDownloader with local instance"""

    def test_is_url_from_local_instance_returns_false_if_url_is_not_from_local_instance(
        self,
    ):
        """test is url from local instance returns false if url is not from local instance

        Returns:

        """
        # Arrange / Act
        return_value = BlobDownloader(
            "http://google.com"
        ).is_url_from_local_instance()
        # Assert
        self.assertEqual(return_value, False)

    def test_is_url_from_local_instance_returns_true_if_url_is_from_local_instance(
        self,
    ):
        """test is url from local instance returns true if url is from local instance

        Returns:

        """
        # Arrange / Act
        return_value = BlobDownloader(
            f"{settings.SERVER_URI}/987653456789"
        ).is_url_from_local_instance()
        # Assert
        self.assertEqual(return_value, True)


class TestBlobDownloaderGetUrlBase(TestCase):
    """Test BlobDownloader.get_url_base"""

    def test_get_url_base_returns_url_base(self):
        """test get url base returns url base

        Returns:

        """
        # Arrange / Act
        return_value = BlobDownloader(
            f"{settings.SERVER_URI}/987653456789"
        ).get_url_base()
        # Assert
        self.assertEqual(return_value, SERVER_URI)

    def test_get_url_base_raise_exception_if_url_cannot_be_parsed(self):
        """test get url base raise exception if url cannot be parsed

        Returns:

        """
        # Assert
        with self.assertRaises(BlobDownloaderUrlParseError):
            # Arrange / Act
            BlobDownloader("random_string").get_url_base()


class TestBlobDownloaderGetBlobResponse(TestCase):
    """Test BlobDownloader.get_blob_response"""

    def test_get_blob_response_raise_exception_if_url_is_unidentified(self):
        """test get blob response raise exception if url is unidentified

        Returns:

        """
        # Assert
        with self.assertRaises(BlobDownloaderUrlParseError):
            # Arrange / Act
            BlobDownloader("random_string").get_blob_response()

    @patch(
        "core_main_app.utils.requests_utils.requests_utils.send_get_request"
    )
    def test_get_blob_response_return_blob_from_local_if_url_is_local(
        self, mock_send_get_request
    ):
        """test get blob response return blob from local if url is local

        Args:
            mock_send_get_request:

        Returns:

        """
        # Arrange / Act
        mock_send_get_request.return_value = "local"
        return_value = BlobDownloader(
            f"{settings.SERVER_URI}/987653456789"
        ).get_blob_response()
        # Assert
        self.assertEqual(return_value, "local")

    @unittest.skip("How to mock the import done in the API?")
    @patch(
        "core_federated_search_app.components.instance.api.get_blob_response_from_url"
    )
    def test_get_blob_response_return_blob_from_remote_if_url_is_identified_as_remote_federated(
        self, mock_send_get_request
    ):
        """test get blob response return blob from remote if url is identified as remote federated

        Args:
            mock_send_get_request:

        Returns:

        """
        # Arrange / Act
        mock_send_get_request.return_value = "remote"
        with patch.object(
            settings, "INSTALLED_APPS", ["core_federated_search_app"]
        ):
            return_value = BlobDownloader(
                "http://my.remote.com/987653456789"
            ).get_blob_response()
        # Assert
        self.assertEqual(return_value, "remote")



""" Tests Settings
"""
from tests.test_settings import *  # noqa


# IN-MEMORY TEST DATABASE
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        "USER": "",
        "PASSWORD": "",
        "HOST": "",
        "PORT": "",
    },
}



""" User Preferences API
"""

from core_main_app.access_control.decorators import access_control
from core_main_app.components.user_preferences.access_control import (
    can_write,
    can_read,
)
from core_main_app.components.user_preferences.models import UserPreferences


@access_control(can_write)
def upsert(user_preferences, user):
    """Save or update the user preferences.

    Args:
        user_preferences:
        user:

    Returns:

    """

    user_preferences.save_object()
    return user_preferences


@access_control(can_write)
def delete(user_preferences, user):
    """Delete the user preferences.

    Args:
        user_preferences:
        user:

    Returns:

    """
    # delete user preferences in database
    return user_preferences.delete()


@access_control(can_read)
def get_by_user(user):
    """Return User Preferences by user.

    Args:
        user: User

    Returns:
        user preferences user id.

    """
    return UserPreferences.get_by_user(user)



"""Query tool unit tests
"""
import copy
import re
from unittest import TestCase

from django.test import override_settings

from core_main_app.commons.exceptions import QueryError
from core_main_app.utils.query.mongo.prepare import (
    _compile_regex,
    _add_sub_document_root,
    sanitize_number,
    sanitize_value,
    convert_to_django,
)


class TestCompileRegex(TestCase):
    """TestCompileRegex"""

    def test_query_with_regex_returns_query_with_compiled_regex(self):
        """test query with regex returns query with compiled regex

        Returns:

        """
        # set query
        query = {"dot.notation": "/regex/"}
        # compile regex
        _compile_regex(query)
        # assert
        self.assertTrue(isinstance(query["dot.notation"], re.Pattern))

    def test_query_with_regex_on_multiple_levels_returns_query_with_compiled_regex(
        self,
    ):
        """test query with regex on multiple levels returns query with compiled regex

        Returns:

        """
        # set query
        query = {
            "$and": [
                {
                    "$or": [
                        {"dot.notation.1": "/regex/"},
                        {"dot.notation.1.#text": "/regex/"},
                    ]
                },
                {
                    "$or": [
                        {"dot.notation.2": {"$lt": 500}},
                        {"dot.notation.2": {"$lt": 500}},
                    ]
                },
            ]
        }
        # compile regex
        _compile_regex(query)
        # assert
        self.assertTrue(
            isinstance(
                query["$and"][0]["$or"][0]["dot.notation.1"], re.Pattern
            )
        )
        self.assertTrue(
            isinstance(
                query["$and"][0]["$or"][1]["dot.notation.1.#text"], re.Pattern
            )
        )
        self.assertFalse(
            isinstance(
                query["$and"][1]["$or"][0]["dot.notation.2"], re.Pattern
            )
        )
        self.assertFalse(
            isinstance(
                query["$and"][1]["$or"][1]["dot.notation.2"], re.Pattern
            )
        )

    def test_query_without_regex_returns_same_query(self):
        """test query without regex returns same query

        Returns:

        """
        # set query
        query = {"dot.notation": {"$gt": 0}}
        # save original query
        query_origin = copy.deepcopy(query)
        # compile regex
        _compile_regex(query)
        # assert
        self.assertEqual(query, query_origin)

    def test_query_without_regex_on_multiple_levels_returns_same_query(self):
        """test query without regex on multiple levels returns same query

        Returns:

        """
        # set query
        query = {
            "$or": [
                {"dot.notation": {"$gt": 0}},
                {"dot.notation.#text": {"$gt": 0}},
            ]
        }
        # save original query
        query_origin = copy.deepcopy(query)
        # compile regex
        _compile_regex(query)
        # assert
        self.assertEqual(query, query_origin)


class TestAddSubDocumentRoot(TestCase):
    """TestAddSubDocumentRoot"""

    def test_add_sub_document_root_to_query(self):
        """test add sub document root to query

        Returns:

        """
        # set query
        query = {"dot.notation": "test"}
        # set expected query
        expected_query = {"root.dot.notation": "test"}
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEqual(query, expected_query)

    def test_add_sub_document_root_to_and_query(self):
        """test add sub document root to and query

        Returns:

        """
        # set query
        query = {
            "$and": [
                {"dot.notation": {"$gt": 0}},
                {"dot.notation.#text": {"$gt": 0}},
            ]
        }
        # set expected query
        expected_query = {
            "$and": [
                {"root.dot.notation": {"$gt": 0}},
                {"root.dot.notation.#text": {"$gt": 0}},
            ]
        }
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEqual(query, expected_query)

    def test_add_sub_document_root_to_or_query(self):
        """test add sub document root to or query

        Returns:

        """
        # set query
        query = {
            "$or": [
                {"dot.notation": {"$gt": 0}},
                {"dot.notation.#text": {"$gt": 0}},
            ]
        }
        # set expected query
        expected_query = {
            "$or": [
                {"root.dot.notation": {"$gt": 0}},
                {"root.dot.notation.#text": {"$gt": 0}},
            ]
        }
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEqual(query, expected_query)

    def test_add_sub_document_root_to_and_or_query(self):
        """test add sub document root to and or query

        Returns:

        """
        # set query
        query = {
            "$and": [
                {
                    "$or": [
                        {"dot.notation.1": "test"},
                        {"dot.notation.1.#text": "/regex/"},
                    ]
                },
                {
                    "$or": [
                        {"dot.notation.2": {"$lt": 500}},
                        {"dot.notation.2": {"$lt": 500}},
                    ]
                },
            ]
        }
        # set expected query
        expected_query = {
            "$and": [
                {
                    "$or": [
                        {"root.dot.notation.1": "test"},
                        {"root.dot.notation.1.#text": "/regex/"},
                    ]
                },
                {
                    "$or": [
                        {"root.dot.notation.2": {"$lt": 500}},
                        {"root.dot.notation.2": {"$lt": 500}},
                    ]
                },
            ]
        }
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEqual(query, expected_query)


class TestSanitizeNumber(TestCase):
    """TestSanitizeNumber"""

    def test_sanitize_number_with_int_is_ok(self):
        """test_sanitize_number_with_int_is_ok

        Returns:

        """
        self.assertTrue(isinstance(sanitize_number(3), int))

    def test_sanitize_number_with_int_str_is_raises_error(self):
        """test_sanitize_number_with_int_str_is_raises_error

        Returns:

        """
        with self.assertRaises(QueryError):
            sanitize_number("3")

    def test_sanitize_number_with_float_is_ok(self):
        """test_sanitize_number_with_float_is_ok

        Returns:

        """
        self.assertTrue(isinstance(sanitize_number(3.0), float))

    def test_sanitize_number_with_float_str_raises_error(self):
        """test_sanitize_number_with_float_str_raises_error

        Returns:

        """
        with self.assertRaises(QueryError):
            sanitize_number("3.0")

    def test_sanitize_number_with_string_raises_error(self):
        """test_sanitize_number_with_string_raises_error

        Returns:

        """
        with self.assertRaises(QueryError):
            sanitize_number("test")


class TestSanitizeValue(TestCase):
    """TestSanitizeValue"""

    def test_sanitize_value_with_str_ok(self):
        """test_sanitize_value_with_str_ok

        Returns:

        """
        self.assertIsNotNone(sanitize_value("test"))

    def test_sanitize_value_with_operator_str_raises_error(self):
        """test_sanitize_value_with_str_ok

        Returns:

        """
        with self.assertRaises(QueryError):
            sanitize_value("$test")

    def test_sanitize_value_with_other_operator_str_raises_error(self):
        """test_sanitize_value_with_str_ok

        Returns:

        """
        with self.assertRaises(QueryError):
            sanitize_value("te$st")

    def test_sanitize_value_without_operator_in_list_ok(self):
        """test_sanitize_value_with_str_ok

        Returns:

        """
        query_list = ["test1", "test2"]
        self.assertEqual(sanitize_value(query_list), query_list)

    def test_sanitize_value_with_operator_in_list_raises_error(self):
        """test_sanitize_value_with_str_ok

        Returns:

        """
        with self.assertRaises(QueryError):
            sanitize_value(["test", "te$st"])


class TestPrepareQuery(TestCase):
    def test_prepare_text_query_for_postgresql(self):
        """test_prepare_text_query_for_postgresql

        Returns:

        """
        from django.db.models import Q

        expected_query = Q(vector_column="test")
        self.assertEqual(
            convert_to_django({"$text": {"$search": "test"}}), expected_query
        )

    @override_settings(
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
            }
        }
    )
    def test_prepare_text_query_for_non_postgresql(self):
        """test_prepare_text_query_for_non_postgresql

        Returns:

        """
        from django.db.models import Q

        expected_query = Q(dict_content__icontains="test")
        self.assertEqual(
            convert_to_django({"$text": {"$search": "test"}}), expected_query
        )

    @override_settings(MONGODB_INDEXING=True)
    def test_prepare_text_query_for_mongodb(self):
        """test_prepare_text_query_for_mongodb

        Returns:

        """
        from mongoengine.queryset.visitor import Q

        expected_query = Q(**{"__raw__": {"$text": {"$search": "test"}}})
        self.assertEqual(
            convert_to_django({"$text": {"$search": "test"}}), expected_query
        )



"""Mock user for tests
"""
from unittest.mock import Mock
from django.contrib.auth.models import User


def create_mock_user(
    user_id,
    is_staff=False,
    is_superuser=False,
    has_perm=False,
    is_anonymous=False,
    is_active=True,
):
    """Mock a User.

    Args:
        user_id:
        is_staff:
        is_superuser:
        has_perm:
        is_anonymous:
        is_active:

    """
    mock_user = Mock(spec=User)
    mock_user.id = user_id
    mock_user.is_staff = is_staff
    mock_user.is_superuser = is_superuser
    mock_user.has_perm.return_value = has_perm
    mock_user.is_anonymous = is_anonymous
    mock_user.is_active = is_active
    mock_user.is_authenticated = True if not is_anonymous else False
    if mock_user.is_anonymous:
        mock_user.id = None
        mock_user.is_active = False

    return mock_user



""" Migrations
"""

# Generated by Django 3.2.10 on 2021-12-17 21:34

import django.contrib.postgres.indexes
import django.contrib.postgres.search
from django.db import migrations

from django.conf import settings
from core_main_app.utils.databases.backend import uses_postgresql_backend


class Migration(migrations.Migration):
    """Migration"""

    dependencies = [
        ("core_main_app", "0002_site_update"),
    ]

    operations = [
        migrations.AddField(
            model_name="data",
            name="vector_column",
            field=django.contrib.postgres.search.SearchVectorField(null=True),
        ),
        migrations.AddIndex(
            model_name="data",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["vector_column"], name="core_main_a_vector__449df8_gin"
            ),
        ),
    ]
    if uses_postgresql_backend() and not settings.MONGODB_INDEXING:
        operations.append(
            migrations.RunSQL(
                sql="""
                  CREATE OR REPLACE FUNCTION update_dict_content_tsvector() RETURNS trigger AS $$
                  BEGIN
                    NEW."vector_column" :=
                      to_tsvector('english', NEW.dict_content::jsonb);
                    RETURN NEW;
                  END;
                  $$ LANGUAGE plpgsql;

                  CREATE TRIGGER vector_column_trigger
                  BEFORE INSERT OR UPDATE OF dict_content, vector_column
                  ON core_main_app_data
                  FOR EACH ROW EXECUTE PROCEDURE
                  update_dict_content_tsvector();
                  UPDATE core_main_app_data SET vector_column = NULL;
                """,
                reverse_sql="""
                  DROP TRIGGER IF EXISTS vector_column_trigger
                  ON core_main_app_data;
                """,
            )
        )



"""
Version Manager API
"""
from core_main_app.access_control.decorators import access_control
from core_main_app.commons import exceptions
from core_main_app.commons.exceptions import ApiError
from core_main_app.components.version_manager.access_control import (
    can_write,
    can_read_version_manager,
)


@access_control(can_write)
def disable(version_manager, request):
    """Disable an object, then saves it.

    Args:
        version_manager:
        request:

    Returns:

    """
    version_manager.is_disabled = True
    version_manager.save()
    return version_manager


@access_control(can_write)
def restore(version_manager, request):
    """Restore an object, then saves it.

    Args:
        version_manager:
        request:

    Returns:

    """
    version_manager.is_disabled = False
    version_manager.save()
    return version_manager


@access_control(can_write)
def restore_version(version, request):
    """Disable a version of the object, then saves it.

    Args:
        version:
        request:

    Returns:

    """
    if not version.is_disabled:
        raise ApiError(
            "Unable to restore this version: status is not disabled."
        )
    version.is_disabled = False
    version.save()


@access_control(can_write)
def disable_version(version, request, new_current=None):
    """Disable a version of the object, then saves it.

    Args:
        version:
        new_current:
        request:

    Returns:

    """
    version_manager = version.version_manager

    # try to delete the current version
    if version.is_current:
        # no version to be current provided
        if new_current is None:
            raise exceptions.ApiError("Unable to disable the current version.")

        # id doesn't match a version
        if new_current.id not in version_manager.versions:
            raise exceptions.ApiError(
                "The id provided to be the next current version, could not be found.",
            )

        # set the new current version
        set_current(new_current, request)

    # disable the version
    version.is_disabled = True
    version.save()


@access_control(can_write)
def set_current(version, request):
    """Set the current version of the object, then saves it.

    Args:
        version:
        request:

    Returns:

    """
    version_manager = version.version_manager

    # a disabled version cannot be current
    if version.is_disabled:
        raise exceptions.ApiError(
            "Unable to set the current version because it is disabled."
        )

    # unset current version
    current_version = version_manager.current_version
    current_version.is_current = False
    current_version.save()

    # set version as current
    version.is_current = True
    version.save()
    return version  # NOTE: use to return version manager


@access_control(can_write)
def upsert(version_manager, request):
    """Save or update version manager.

    Args:
        version_manager:
        request:

    Returns:

    """
    return version_manager.save_version_manager()


@access_control(can_read_version_manager)
def get_version_number(version_manager, version_id, request):
    """Return version number from version id.

    Args:
        version_manager:
        version_id:
        request:

    Returns:

    """
    return list(version_manager.versions).index(str(version_id)) + 1


@access_control(can_read_version_manager)
def get_version_by_number(version_manager, version_number, request):
    """Return the version by its version number.

    Args:
        version_manager:
        version_number: Number of the version
        request:

    Returns:

    """
    return version_manager.versions[version_number - 1]



""" Fixtures files for Data
"""
from django.core.files.uploadedfile import SimpleUploadedFile

from core_main_app.components.blob.models import Blob
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.components.workspace import api as workspace_api
from core_main_app.components.workspace.models import Workspace
from core_main_app.components.xsl_transformation.models import (
    XslTransformation,
)
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)


class DataFixtures(FixtureInterface):
    """Data fixtures"""

    data_1 = None
    data_2 = None
    data_3 = None
    template = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.data_1 = Data(
            template=self.template,
            user_id="1",
            dict_content=None,
            title="title",
            xml_content="<root></root>",
        )
        self.data_1.save()
        self.data_2 = Data(
            template=self.template,
            user_id="2",
            dict_content=None,
            title="title2",
            xml_content="<root></root>",
        )
        self.data_2.save()
        self.data_3 = Data(
            template=self.template,
            user_id="1",
            dict_content=None,
            title="title3",
            xml_content="<root></root>",
        )
        self.data_3.save()
        self.data_collection = [self.data_1, self.data_2, self.data_3]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()


class QueryDataFixtures(DataFixtures):
    """Data fixtures"""

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        content_1 = {
            "root": {
                "element": "value",
                "list": [{"element_list_1": 1}, {"element_list_2": 2}],
                "complex": {"child1": "test", "child2": 0},
            }
        }
        content_2 = {"root": {"element": "value2"}}
        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.data_1 = Data(
            template=self.template,
            user_id="1",
            dict_content=content_1,
            title="title",
            xml_content="<root>"
            "<element>value</element>"
            "<list><element_list_1>1</element_list_1></list>"
            "<list><element_list_2>2</element_list_2></list>"
            "<complex><child1>test</child1><child2>0</child2></complex>"
            "</root>",
        )
        self.data_1.save()
        self.data_2 = Data(
            template=self.template,
            user_id="2",
            dict_content=content_2,
            title="title2",
            xml_content="<root><element>value</element></root>",
        )
        self.data_2.save()
        self.data_collection = [self.data_1, self.data_2]


class AccessControlDataFixture(FixtureInterface):
    """Access Control Data fixture"""

    USER_1_NO_WORKSPACE = 0
    USER_2_NO_WORKSPACE = 1
    USER_1_WORKSPACE_1 = 2
    USER_2_WORKSPACE_2 = 3

    template = None
    workspace_1 = None
    workspace_2 = None
    data_collection = None
    data_1 = None
    data_2 = None
    data_3 = None
    data_4 = None
    data_5 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspace()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        xml_content = "<root><element>value2</element></root>"
        content = {"root": {"element": "value2"}}

        self.data_1 = Data(
            template=self.template,
            title="Data 1",
            user_id="1",
            xml_content="<root></root>",
        )
        self.data_1.save()
        self.data_2 = Data(
            template=self.template,
            title="Data 2",
            user_id="2",
            xml_content="<root></root>",
        )
        self.data_2.save()
        self.data_3 = Data(
            template=self.template,
            title="Data 3",
            user_id="1",
            workspace=self.workspace_1,
            xml_content=xml_content,
            dict_content=content,
        )
        self.data_3.save()
        self.data_4 = Data(
            template=self.template,
            title="DataDoubleTitle",
            user_id="2",
            workspace=self.workspace_2,
            xml_content="<root></root>",
        )
        self.data_4.save()
        self.data_5 = Data(
            template=self.template,
            title="DataDoubleTitle",
            user_id="1",
            workspace=self.workspace_1,
            xml_content=xml_content,
        )
        self.data_5.save()
        self.data_collection = [
            self.data_1,
            self.data_2,
            self.data_3,
            self.data_4,
            self.data_5,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()

    def generate_workspace(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_1 = Workspace(
            title="Workspace 1", owner="1", read_perm_id="1", write_perm_id="1"
        )
        self.workspace_1.save()
        self.workspace_2 = Workspace(
            title="Workspace 2", owner="2", read_perm_id="2", write_perm_id="2"
        )
        self.workspace_2.save()

    def generate_workspace_with_perm(self):
        """Generate the workspaces and the perm object.

        Returns:

        """
        try:
            self.workspace_1 = workspace_api.create_and_save("Workspace 1")
            self.workspace_2 = workspace_api.create_and_save("Workspace 2")
            self.data_3.workspace = self.workspace_1
            self.data_4.workspace = self.workspace_2
            self.data_5.workspace = self.workspace_1
        except Exception as exception:
            print(str(exception))


class AccessControlDataFixture2(FixtureInterface):
    """Access Control Data fixture
    - User1: 1 private data
    - User2: 1 data in workspace 1
    - User3: 1 private data and 1 data in workspace 1
    """

    template = None
    workspace_1 = None
    data_collection = None
    data_1 = None
    data_2 = None
    data_3_1 = None
    data_3_2 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspace()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        xml_content = "<root><element>value2</element></root>"
        content = {"root": {"element": "value2"}}

        self.data_1 = Data(
            template=self.template,
            title="Data 1",
            user_id="1",
            xml_content=xml_content,
            dict_content=content,
        )
        self.data_1.save()
        self.data_2 = Data(
            template=self.template,
            title="Data 2",
            user_id="2",
            xml_content="<root></root>",
            workspace=self.workspace_1,
        )
        self.data_2.save()
        self.data_3_1 = Data(
            template=self.template,
            title="Data 3.1",
            user_id="3",
            xml_content=xml_content,
            dict_content=content,
        )
        self.data_3_1.save()
        self.data_3_2 = Data(
            template=self.template,
            title="Data 3.2",
            user_id="3",
            workspace=self.workspace_1,
            xml_content=xml_content,
            dict_content=content,
        )
        self.data_3_2.save()
        self.data_collection = [
            self.data_1,
            self.data_2,
            self.data_3_1,
            self.data_3_2,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()

    def generate_workspace(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_1 = Workspace(
            title="Workspace 1",
            owner=None,
            read_perm_id="1",
            write_perm_id="1",
        )
        self.workspace_1.save()


class AccessControlDataFullTextSearchFixture(FixtureInterface):
    """Access Control Data fixture
    - User1: 1 private data
    - User2: 1 data in workspace 1
    - User3: 1 private data and 1 data in workspace 1
    """

    template = None
    workspace_1 = None
    data_collection = None
    data_1 = None
    data_2 = None
    data_3_1 = None
    data_3_2 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspace()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        # _1: user1 is unique and private, value1 is also in _2
        xml_content_1 = (
            "<root><element>user1</element><element>value1</element></root>"
        )
        # _2: value1 is common to _1 and _2
        xml_content_2 = "<root><element>value1</element></root>"
        # _3_1: value31 is unique and private
        xml_content_3_1 = "<root><element>value31</element></root>"
        # _3_2: value32 is unique
        xml_content_3_2 = "<root><element>value32</element></root>"

        self.data_1 = Data(
            template=self.template,
            title="Data 1",
            user_id="1",
            xml_content=xml_content_1,
        )
        self.data_1.convert_and_save()
        self.data_2 = Data(
            template=self.template,
            title="Data 2",
            user_id="2",
            xml_content=xml_content_2,
            workspace=self.workspace_1,
        )
        self.data_2.convert_and_save()
        self.data_3_1 = Data(
            template=self.template,
            title="Data 3.1",
            user_id="3",
            xml_content=xml_content_3_1,
        )
        self.data_3_1.convert_and_save()
        self.data_3_2 = Data(
            template=self.template,
            title="Data 3.2",
            user_id="3",
            workspace=self.workspace_1,
            xml_content=xml_content_3_2,
        )
        self.data_3_2.convert_and_save()
        self.data_collection = [
            self.data_1,
            self.data_2,
            self.data_3_1,
            self.data_3_2,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()

    def generate_workspace(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_1 = Workspace(
            title="Workspace 1",
            owner=None,
            read_perm_id="1",
            write_perm_id="1",
        )
        self.workspace_1.save()


class AccessControlDataNumericFixture(FixtureInterface):
    """Access Control Data fixture
    - User1: 1 private data
    - User2: 1 data in workspace 1
    - User3: 1 private data and 1 data in workspace 1
    """

    template = None
    workspace_1 = None
    data_collection = None
    data_1 = None
    data_2 = None
    data_3_1 = None
    data_3_2 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspace()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        # value: 1
        xml_content_1 = "<root><element>1</element></root>"
        # value: 2
        xml_content_2 = "<root><element>2</element></root>"
        # value: 31
        xml_content_3_1 = "<root><element>31</element></root>"
        # value: 32
        xml_content_3_2 = "<root><element>32</element></root>"

        self.data_1 = Data(
            template=self.template,
            title="Data 1",
            user_id="1",
            xml_content=xml_content_1,
        )
        self.data_1.convert_and_save()
        self.data_2 = Data(
            template=self.template,
            title="Data 2",
            user_id="2",
            xml_content=xml_content_2,
            workspace=self.workspace_1,
        )
        self.data_2.convert_and_save()
        self.data_3_1 = Data(
            template=self.template,
            title="Data 3.1",
            user_id="3",
            xml_content=xml_content_3_1,
        )
        self.data_3_1.convert_and_save()
        self.data_3_2 = Data(
            template=self.template,
            title="Data 3.2",
            user_id="3",
            workspace=self.workspace_1,
            xml_content=xml_content_3_2,
        )
        self.data_3_2.convert_and_save()
        self.data_collection = [
            self.data_1,
            self.data_2,
            self.data_3_1,
            self.data_3_2,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()

    def generate_workspace(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_1 = Workspace(
            title="Workspace 1",
            owner=None,
            read_perm_id="1",
            write_perm_id="1",
        )
        self.workspace_1.save()


class AccessControlDataNoneFixture(FixtureInterface):
    """Access Control Data fixture
    - User1: 1 private data
    - User2: 1 data in workspace 1
    """

    template = None
    workspace_1 = None
    data_collection = None
    data_1 = None
    data_2 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspace()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        # value: 1
        xml_content_1 = "<root><element></element></root>"
        # value: 2
        xml_content_2 = "<root><element>value</element></root>"

        self.data_1 = Data(
            template=self.template,
            title="Data 1",
            user_id="1",
            xml_content=xml_content_1,
        )
        self.data_1.convert_and_save()
        self.data_2 = Data(
            template=self.template,
            title="Data 2",
            user_id="2",
            xml_content=xml_content_2,
            workspace=self.workspace_1,
        )
        self.data_2.convert_and_save()
        self.data_collection = [
            self.data_1,
            self.data_2,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()

    def generate_workspace(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_1 = Workspace(
            title="Workspace 1",
            owner=None,
            read_perm_id="1",
            write_perm_id="1",
        )
        self.workspace_1.save()


class DataMigrationFixture(FixtureInterface):
    """Data Template Fixture"""

    template_1 = None
    template_2 = None
    template_3 = None
    template_4 = None
    data_collection = None
    data_1 = None
    data_2 = None
    data_3 = None
    data_4 = None
    data_5 = None
    xsl_transformation = None
    xsl_transformation_2 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        self.data_1 = Data(
            template=self.template_1, title="Data 1", user_id="1"
        )
        self.data_1.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <test>test</test> \
                                  </root>'
        self.data_1.save()

        self.data_2 = Data(
            template=self.template_1, title="Data 2", user_id="1"
        )
        self.data_2.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <test>test</test> \
                                  </root>'
        self.data_2.save()

        self.data_3 = Data(
            template=self.template_2, title="Data 3", user_id="1"
        )
        self.data_3.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <test>test</test> \
                                  </root>'

        self.data_3.save()

        self.data_4 = Data(
            template=self.template_3, title="Data4", user_id="1"
        )
        self.data_4.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <other>test</other> \
                                  </root>'
        self.data_4.save()

        self.data_5 = Data(
            template=self.template_3, title="Data5", user_id="1"
        )
        self.data_5.xml_content = '<root xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"> \
                                    <other>test</other> \
                                  </root>'
        self.data_5.save()

        self.data_collection = [
            self.data_1,
            self.data_2,
            self.data_3,
            self.data_4,
            self.data_5,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template_1 = Template()
        self.template_2 = Template()
        self.template_3 = Template()
        self.template_4 = Template()
        xsd1 = '<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"> \
                <xsd:element name="root" type="simpleString"/> \
                <xsd:complexType name="simpleString"> \
                    <xsd:sequence> \
                    <xsd:element name="test" type="xsd:string"/></xsd:sequence> \
                </xsd:complexType> \
            </xsd:schema>'
        xsd2 = '<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"> \
                <xsd:element name="root" type="simpleString"/> \
                <xsd:complexType name="simpleString"> \
                    <xsd:sequence> \
                    <xsd:element name="test" type="xsd:string"/></xsd:sequence> \
                </xsd:complexType> \
            </xsd:schema>'
        xsd3 = '<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"> \
                <xsd:element name="root" type="simpleString"/> \
                <xsd:complexType name="simpleString"> \
                    <xsd:sequence> \
                    <xsd:element name="other" type="xsd:string"/></xsd:sequence> \
                </xsd:complexType> \
            </xsd:schema>'
        xsd4 = '<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"> \
                        <xsd:element name="root" type="simpleString" /> \
                        <xsd:complexType name="simpleString"> \
                            <xsd:sequence> \
                                <xsd:element name="test" type="xsd:string"/> \
                                <xsd:element name="element" type="xsd:string"/> \
                            </xsd:sequence> \
                        </xsd:complexType> \
                </xsd:schema>'
        self.template_1.content = xsd1
        self.template_1.hash = ""
        self.template_1.filename = "filename"
        self.template_2.content = xsd2
        self.template_2.hash = ""
        self.template_2.filename = "filename"
        self.template_3.content = xsd3
        self.template_3.hash = ""
        self.template_3.filename = "filename"
        self.template_4.content = xsd4
        self.template_4.hash = ""
        self.template_4.filename = "filename"
        self.template_1.save_template()
        self.template_2.save_template()
        self.template_3.save_template()
        self.template_4.save_template()

    def generate_xslt(self):
        """Generate xsl transformation .

        Returns:

        """

        content = '<?xml version="1.0" encoding="UTF-8"?> \
                <xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"> \
                    <xsl:template match="@* | node()">   \
                        <xsl:copy> \
                            <xsl:apply-templates select="@* | node()"/> \
                        </xsl:copy> \
                    </xsl:template> \
                    <xsl:template match="test"> \
                        <xsl:copy-of select="."/> \
                        <element>new_element</element> \
                    </xsl:template> \
                </xsl:stylesheet>'
        self.xsl_transformation = XslTransformation(
            name="xsl_transformation",
            filename="xsl_transformation.xsl",
            file=SimpleUploadedFile(
                "xsl_transformation.xsl", content=content.encode("utf-8")
            ),
        )

        self.xsl_transformation.save()


class AccessControlBlobWithMetadataFixture(FixtureInterface):
    """Access Control Blob with Metadata Fixture
    blob1 (user: 1, wksp: None)
        |-data1 (user: 1, wksp: None)

    blob2 (user: 2, wksp: 1)
        |-data2 (user: 2, wksp: 1)

    blob3 (user: 3, wksp: 1)
        |-data31 (user: 3, wksp: 1)
        |-data32 (user: 3, wksp: None)

    data4 (user: 1, wksp: None)
    """

    template = None
    workspace_1 = None
    data_collection = None
    blob_collection = None

    data_1 = None
    data_2 = None
    data_31 = None
    data_32 = None
    data_4 = None

    blob_1 = None
    blob_2 = None
    blob_3 = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_template()
        self.generate_workspace()
        self.generate_blob_collection()
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """

        xml_content = "<root><element>value2</element></root>"
        content = {"root": {"element": "value2"}}

        self.data_1 = Data(
            template=self.template,
            title="Data 1",
            user_id="1",
            _blob=self.blob_1,
            xml_content="<root></root>",
        )
        self.data_1.save()
        self.data_2 = Data(
            template=self.template,
            title="Data 2",
            user_id="2",
            _blob=self.blob_2,
            workspace=self.workspace_1,
            xml_content="<root></root>",
        )
        self.data_2.save()
        self.data_31 = Data(
            template=self.template,
            title="Data 3_1",
            user_id="3",
            workspace=self.workspace_1,
            xml_content=xml_content,
            dict_content=content,
            _blob=self.blob_3,
        )
        self.data_31.save()
        self.data_32 = Data(
            template=self.template,
            title="Data 3_2",
            user_id="3",
            xml_content="<root></root>",
            _blob=self.blob_3,
        )
        self.data_32.save()
        self.data_4 = Data(
            template=self.template,
            title="Data 4",
            user_id="1",
            xml_content="<root></root>",
        )
        self.data_4.save()

        self.data_collection = [
            self.data_1,
            self.data_2,
            self.data_31,
            self.data_32,
            self.data_4,
        ]

    def generate_template(self):
        """Generate an unique Template.

        Returns:

        """
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="tag"></xs:element></xs:schema>'
        )
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()

    def generate_workspace(self):
        """Generate the workspaces.

        Returns:

        """
        self.workspace_1 = Workspace(
            title="Workspace 1", owner="1", read_perm_id="1", write_perm_id="1"
        )
        self.workspace_1.save()

    def generate_blob_collection(self):
        """Generate a Blob collection.

        Returns:

            user 1 -> blob1, blob2
            user 2 -> blob3

        """

        self.blob_1 = Blob(
            filename="blob1",
            user_id="1",
            blob=SimpleUploadedFile("blob.txt", b"blob"),
        )
        self.blob_1.save()
        self.blob_2 = Blob(
            filename="blob2",
            user_id="2",
            workspace=self.workspace_1,
            blob=SimpleUploadedFile("blob.txt", b"blob"),
        )
        self.blob_2.save()
        self.blob_3 = Blob(
            filename="blob3",
            user_id="3",
            workspace=self.workspace_1,
            blob=SimpleUploadedFile("blob.txt", b"blob"),
        )
        self.blob_3.save()

        self.blob_collection = [self.blob_1, self.blob_2, self.blob_3]



""" Custom admin site for the Template model
"""
from django.contrib import admin


class CustomTemplateAdmin(admin.ModelAdmin):
    """CustomTemplateAdmin"""

    readonly_fields = ["checksum", "hash", "file"]
    exclude = ["_cls"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Templates"""
        return False



""" Authentication tests for Template Version Manager REST API
"""
from unittest.mock import patch

from core_main_app.access_control.exceptions import AccessControlError
from django.test import SimpleTestCase
from rest_framework import status

from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.rest.template_version_manager import (
    views as template_version_manager_views,
)
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.rest.template_version_manager.serializers import (
    TemplateVersionManagerSerializer,
    CreateTemplateSerializer,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGlobalTemplateVersionManagerListGetPermission(SimpleTestCase):
    """TestGlobalTemplateVersionManagerListGetPermission"""

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_anonymous_returns_http_403(
        self, template_version_manager_get_all_global_version_managers
    ):
        """test_anonymous_returns_http_403

        Args:
            template_version_manager_get_all_global_version_managers:

        Returns:

        """
        template_version_manager_get_all_global_version_managers.return_value = TemplateVersionManager(
            user=None
        )

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_authenticated_returns_http_200(
        self, template_version_manager_get_all_global_version_managers
    ):
        """test_authenticated_returns_http_200

        Args:
            template_version_manager_get_all_global_version_managers:

        Returns:

        """
        template_version_manager_get_all_global_version_managers.return_value = (
            {}
        )

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_global_version_managers")
    def test_staff_returns_http_200(
        self, template_version_manager_get_all_global_version_managers
    ):
        """test_staff_returns_http_200

        Args:
            template_version_manager_get_all_global_version_managers:

        Returns:

        """
        template_version_manager_get_all_global_version_managers.return_value = (
            {}
        )

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserTemplateVersionManagerListGetPermission(SimpleTestCase):
    """TestUserTemplateVersionManagerListGetPermission"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateVersionManager, "get_all_version_manager_by_user_id")
    def test_authenticated_returns_http_200(
        self, template_version_manager_get_all_version_manager_by_user_id
    ):
        """test_authenticated_returns_http_200

        Args:
            template_version_manager_get_all_version_manager_by_user_id:

        Returns:

        """
        template_version_manager_get_all_version_manager_by_user_id.return_value = (
            {}
        )

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_all_version_manager_by_user_id")
    def test_staff_returns_http_200(
        self, template_version_manager_get_all_version_manager_by_user_id
    ):
        """test_staff_returns_http_200

        Args:
            template_version_manager_get_all_version_manager_by_user_id:

        Returns:

        """
        template_version_manager_get_all_version_manager_by_user_id.return_value = (
            {}
        )

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionManagerDetailGetPermission(SimpleTestCase):
    """TestTemplateVersionManagerDetailGetPermission"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateVersionManager, "get_by_id")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_authenticated_returns_http_200(
        self, template_version_manager_data, version_manager_get_by_id
    ):
        """test_authenticated_returns_http_200

        Args:
            template_version_manager_data:
            version_manager_get_by_id:

        Returns:

        """
        template_version_manager_data.return_value = True
        version_manager_get_by_id.return_value = TemplateVersionManager(
            user="1"
        )

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_by_id")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_200(
        self, template_version_manager_data, version_manager_get_by_id
    ):
        """test_staff_returns_http_200

        Args:
            template_version_manager_data:
            version_manager_get_by_id:

        Returns:

        """
        template_version_manager_data.return_value = True
        version_manager_get_by_id.return_value = TemplateVersionManager(
            user="1"
        )

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.TemplateVersionManagerDetail.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionPostPermission(SimpleTestCase):
    """TestTemplateVersionPostPermission"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateVersionManager, "get_by_id")
    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    def test_staff_returns_http_201(
        self,
        template_version_manager_serializer_data,
        template_version_manager_serializer_save,
        template_version_manager_serializer_valid,
        version_manager_get_by_id,
    ):
        """test_staff_returns_http_201

        Args:
            template_version_manager_serializer_data:
            template_version_manager_serializer_save:
            template_version_manager_serializer_valid:
            version_manager_get_by_id:

        Returns:

        """
        version_manager_get_by_id.return_value = TemplateVersionManager(
            user=None
        )
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.TemplateVersion.as_view(),
            mock_user,
            data={},
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestUserTemplateListPostPermission(SimpleTestCase):
    """TestUserTemplateListPostPermission"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_authenticated_returns_http_201(
        self,
        template_version_manager_serializer_data,
        template_version_manager_serializer_save,
        template_version_manager_serializer_valid,
        create_template_version_manager_serializer_data,
        create_template_version_manager_serializer_save,
        create_template_version_manager_serializer_valid,
    ):
        """test_authenticated_returns_http_201

        Args:
            template_version_manager_serializer_data:
            template_version_manager_serializer_save:
            template_version_manager_serializer_valid:
            create_template_version_manager_serializer_data:
            create_template_version_manager_serializer_save:
            create_template_version_manager_serializer_valid:

        Returns:

        """
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_201(
        self,
        template_version_manager_serializer_data,
        template_version_manager_serializer_save,
        template_version_manager_serializer_valid,
        create_template_version_manager_serializer_data,
        create_template_version_manager_serializer_save,
        create_template_version_manager_serializer_valid,
    ):
        """test_staff_returns_http_201

        Args:
            template_version_manager_serializer_data:
            template_version_manager_serializer_save:
            template_version_manager_serializer_valid:
            create_template_version_manager_serializer_data:
            create_template_version_manager_serializer_save:
            create_template_version_manager_serializer_valid:

        Returns:

        """
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.UserTemplateList.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestGlobalTemplateListPostPermission(SimpleTestCase):
    """TestGlobalTemplateListPostPermission"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(CreateTemplateSerializer, "is_valid")
    @patch.object(CreateTemplateSerializer, "save")
    @patch.object(CreateTemplateSerializer, "data")
    @patch.object(TemplateVersionManagerSerializer, "is_valid")
    @patch.object(TemplateVersionManagerSerializer, "save")
    @patch.object(TemplateVersionManagerSerializer, "data")
    def test_staff_returns_http_201(
        self,
        template_version_manager_serializer_data,
        template_version_manager_serializer_save,
        template_version_manager_serializer_valid,
        create_template_version_manager_serializer_data,
        create_template_version_manager_serializer_save,
        create_template_version_manager_serializer_valid,
    ):
        """test_staff_returns_http_201

        Args:
            template_version_manager_serializer_data:
            template_version_manager_serializer_save:
            template_version_manager_serializer_valid:
            create_template_version_manager_serializer_data:
            create_template_version_manager_serializer_save:
            create_template_version_manager_serializer_valid:

        Returns:

        """
        create_template_version_manager_serializer_data.return_value = True
        create_template_version_manager_serializer_save.return_value = None
        create_template_version_manager_serializer_valid.return_value = {}
        template_version_manager_serializer_data.return_value = True
        template_version_manager_serializer_save.return_value = None
        template_version_manager_serializer_valid.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            template_version_manager_views.GlobalTemplateList.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestCurrentTemplateVersionPatchPermission(SimpleTestCase):
    """TestCurrentTemplateVersionPatchPermission"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            mock_user,
            data={},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.version_manager.api.set_current")
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(
        self,
        template_get_by_id,
        version_manager_set_current,
    ):
        """test_staff_returns_http_200

        Args:
            template_get_by_id:
            version_manager_set_current:

        Returns:

        """
        version_manager_set_current.return_value = {}
        # version_manager_get_from_version.return_value = VersionManager(user="1")
        template_get_by_id.return_value = Template(user="1")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.CurrentTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDisableTemplateVersionPatchPermission(SimpleTestCase):
    """TestDisableTemplateVersionPatchPermission"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.version_manager.api.disable_version")
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(
        self,
        template_get_by_id,
        version_manager_disable_version,
    ):
        """test_staff_returns_http_200

        Args:
            template_get_by_id:
            version_manager_disable_version:

        Returns:

        """
        version_manager_disable_version.return_value = TemplateVersionManager(
            user="1"
        )
        template_get_by_id.return_value = Template(user="1")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRestoreTemplateVersionPatchPermission(SimpleTestCase):
    """TestRestoreTemplateVersionPatchPermission"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Returns:

        """
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.version_manager.api.restore_version")
    @patch.object(Template, "get_by_id")
    def test_staff_returns_http_200(
        self,
        template_get_by_id,
        version_manager_restore_version,
    ):
        """test_staff_returns_http_200

        Args:
            template_get_by_id:
            version_manager_restore_version:

        Returns:

        """
        version_manager_restore_version.return_value = {}
        template_get_by_id.return_value = Template(user="1")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersion.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestDisableTemplateVersionManagerPatchPermission(SimpleTestCase):
    """TestDisableTemplateVersionManagerPatchPermission"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(TemplateVersionManager, "get_by_id")
    @patch("core_main_app.components.version_manager.api.disable")
    def test_authenticated_returns_http_200(
        self, version_manager_disable, version_manager_get_by_id
    ):
        """test_authenticated_returns_http_200

        Args:
            version_manager_disable:
            version_manager_get_by_id:

        Returns:

        """
        version_manager_get_by_id.return_value = TemplateVersionManager(
            user="1"
        )
        version_manager_disable.return_value = {}
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(TemplateVersionManager, "get_by_id")
    @patch("core_main_app.components.version_manager.api.disable")
    def test_staff_returns_http_200(
        self, version_manager_disable, version_manager_get_by_id
    ):
        """test_staff_returns_http_200

        Args:
            version_manager_disable:
            version_manager_get_by_id:

        Returns:

        """
        version_manager_get_by_id.return_value = TemplateVersionManager(
            user="1"
        )
        version_manager_disable.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.DisableTemplateVersionManager.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRestoreTemplateVersionManagerPatchPermission(SimpleTestCase):
    """TestRestoreTemplateVersionManagerPatchPermission"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            None,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("core_main_app.components.version_manager.api.restore")
    @patch.object(TemplateVersionManager, "get_by_id")
    def test_authenticated_returns_http_200(
        self, version_manager_get_by_id, version_manager_restore
    ):
        """test_authenticated_returns_http_200

        Args:
            version_manager_get_by_id:
            version_manager_restore:

        Returns:

        """
        version_manager_restore.return_value = {}
        version_manager_get_by_id.return_value = TemplateVersionManager(
            user="1"
        )
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch("core_main_app.components.version_manager.api.restore")
    @patch.object(TemplateVersionManager, "get_by_id")
    def test_staff_returns_http_200(
        self, version_manager_get_by_id, version_manager_restore
    ):
        """test_staff_returns_http_200

        Args:
            version_manager_get_by_id:
            version_manager_restore:

        Returns:

        """
        version_manager_restore.return_value = {}
        version_manager_get_by_id.return_value = TemplateVersionManager(
            user="1"
        )

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            template_version_manager_views.RestoreTemplateVersionManager.as_view(),
            mock_user,
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionManagerListGetPermission(SimpleTestCase):
    """TestGlobalTemplateVersionManagerListGetPermission"""

    @patch.object(template_version_manager_api, "get_all")
    def test_anonymous_returns_http_403(
        self, template_version_manager_get_all
    ):
        """test_anonymous_returns_http_403

        Args:
            template_version_manager_get_all:

        Returns:

        """
        template_version_manager_get_all.return_value = TemplateVersionManager(
            user=None
        )

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalAndUserTemplateVersionManagerList.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(template_version_manager_api, "get_all")
    def test_authenticated_returns_http_403(
        self, template_version_manager_get_all
    ):
        """test_authenticated_returns_http_200

        Args:
            template_version_manager_get_all:

        Returns:

        """
        template_version_manager_get_all.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalAndUserTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(template_version_manager_api, "get_all")
    def test_staff_returns_http_200(self, template_version_manager_get_all):
        """test_staff_returns_http_200

        Args:
            template_version_manager_get_all:

        Returns:

        """
        template_version_manager_get_all.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalAndUserTemplateVersionManagerList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestGlobalTemplateVersionManagerOrderingListGetPermission(
    SimpleTestCase
):
    """Test Template Version Manager Ordering List Get Permission"""

    def test_anonymous_returns_http_403(
        self,
    ):
        """test_anonymous_returns_http_403

        Args:

        Returns:

        """
        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerOrdering.as_view(),
            None,
        )
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403

        Args:

        Returns:

        """

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerOrdering.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(template_version_manager_api, "get_global_version_managers")
    def test_staff_returns_http_200(self, template_version_manager_get_all):
        """test_staff_returns_http_200

        Args:
            template_version_manager_get_all:

        Returns:

        """
        template_version_manager_get_all.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerOrdering.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(template_version_manager_api, "get_global_version_managers")
    def test_superuser_returns_http_200(
        self, template_version_manager_get_all
    ):
        """test_superuser_returns_http_200

        Args:
            template_version_manager_get_all:

        Returns:

        """
        template_version_manager_get_all.return_value = {}

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.GlobalTemplateVersionManagerOrdering.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestGlobalTemplateVersionManagerOrderingPatchPermission(SimpleTestCase):
    """Test Template Version Manager Ordering Patch Permission"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            None,
        )
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch(
        "core_main_app.components.template_version_manager.api.get_global_version_managers"
    )
    def test_authenticated_returns_http_403(
        self, mock_get_global_version_managers
    ):
        """test_staff_returns_http_403

        Args:


        Returns:

        """
        mock_get_global_version_managers.side_effect = AccessControlError("")
        response = RequestMock.do_request_patch(
            template_version_manager_views.GlobalTemplateVersionManagerOrdering.as_view(),
            create_mock_user("1"),
            data={"template_list": []},
        )
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch(
        "core_main_app.components.template_version_manager.api.get_global_version_managers"
    )
    @patch(
        "core_main_app.components.template_version_manager.api.update_global_template_ordering"
    )
    def test_staff_returns_http_200(
        self,
        mock_get_global_version_managers,
        mock_update_templates_ordering,
    ):
        """test_staff_returns_http_200

        Args:
            mock_get_global_version_managers:
            mock_update_templates_ordering:

        Returns:

        """
        mock_user = create_mock_user("1", is_staff=True)
        mock_get_global_version_managers.return_value = []
        mock_update_templates_ordering.return_value = []
        response = RequestMock.do_request_patch(
            template_version_manager_views.GlobalTemplateVersionManagerOrdering.as_view(),
            mock_user,
            data={"template_list": []},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch(
        "core_main_app.components.template_version_manager.api.get_global_version_managers"
    )
    @patch(
        "core_main_app.components.template_version_manager.api.update_global_template_ordering"
    )
    def test_superuser_returns_http_200(
        self,
        mock_get_global_version_managers,
        mock_update_templates_ordering,
    ):
        """test_superuser_returns_http_200

        Args:
            mock_get_global_version_managers:
            mock_update_templates_ordering:

        Returns:

        """
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)
        mock_get_global_version_managers.return_value = []
        mock_update_templates_ordering.return_value = []
        response = RequestMock.do_request_patch(
            template_version_manager_views.GlobalTemplateVersionManagerOrdering.as_view(),
            mock_user,
            data={"template_list": []},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserTemplateVersionManagerOrderingListGetPermission(SimpleTestCase):
    """Test Template Version Manager Ordering List Get Permission"""

    def test_anonymous_returns_http_403(
        self,
    ):
        """test_anonymous_returns_http_403

        Args:

        Returns:

        """
        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            None,
        )
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(template_version_manager_api, "get_all_by_user_id")
    def test_authenticated_returns_http_200(
        self, template_version_manager_get_all_by_user_id
    ):
        """test_authenticated_returns_http_200

        Args:
            template_version_manager_get_all_by_user_id:

        Returns:

        """
        template_version_manager_get_all_by_user_id.return_value = {}

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(template_version_manager_api, "get_all_by_user_id")
    def test_staff_returns_http_200(self, template_version_manager_get_all):
        """test_staff_returns_http_200

        Args:
            template_version_manager_get_all:

        Returns:

        """
        template_version_manager_get_all.return_value = {}

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(template_version_manager_api, "get_all_by_user_id")
    def test_superuser_returns_http_200(
        self, template_version_manager_get_all
    ):
        """test_superuser_returns_http_200

        Args:
            template_version_manager_get_all:

        Returns:

        """
        template_version_manager_get_all.return_value = {}

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_get(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserTemplateVersionManagerOrderingPatchPermission(SimpleTestCase):
    """Test Template Version Manager Ordering Patch Permission"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_patch(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            None,
        )
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch(
        "core_main_app.components.template_version_manager.api.get_all_by_user_id"
    )
    @patch(
        "core_main_app.components.template_version_manager.api.update_user_template_ordering"
    )
    def test_authenticated_returns_http_200(
        self,
        mock_get_all_by_user_id,
        mock_update_templates_ordering,
    ):
        """test_staff_returns_http_200

        Args:
            mock_get_all_by_user_id:
            mock_update_templates_ordering:

        Returns:

        """
        mock_user = create_mock_user("1")
        mock_get_all_by_user_id.return_value = []
        mock_update_templates_ordering.return_value = []
        response = RequestMock.do_request_patch(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            mock_user,
            data={"template_list": []},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch(
        "core_main_app.components.template_version_manager.api.get_by_id_list"
    )
    @patch(
        "core_main_app.components.template_version_manager.api.update_user_template_ordering"
    )
    @patch.object(template_version_manager_api, "get_all_by_user_id")
    def test_staff_returns_http_200(
        self,
        mock_get_by_user_id,
        mock_update_templates_ordering,
        mock_get_all_by_user_id,
    ):
        """test_staff_returns_http_200

        Args:
            mock_get_all_by_user_id:
            mock_update_templates_ordering:

        Returns:

        """
        mock_user = create_mock_user("1", is_staff=True)
        mock_get_all_by_user_id.return_value = []
        mock_get_by_user_id.return_value = []
        mock_update_templates_ordering.return_value = []
        response = RequestMock.do_request_patch(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            mock_user,
            data={"template_list": []},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch(
        "core_main_app.components.template_version_manager.api.get_by_id_list"
    )
    @patch(
        "core_main_app.components.template_version_manager.api.update_user_template_ordering"
    )
    @patch.object(template_version_manager_api, "get_all_by_user_id")
    def test_superuser_returns_http_200(
        self,
        mock_get_by_user_id,
        mock_update_templates_ordering,
        mock_get_all_by_user_id,
    ):
        """test_superuser_returns_http_200

        Args:
            mock_get_all_by_user_id:
            mock_update_templates_ordering:

        Returns:

        """
        mock_user = create_mock_user("1", is_superuser=True)
        mock_get_all_by_user_id.return_value = []
        mock_get_by_user_id.return_value = []
        mock_update_templates_ordering.return_value = []
        response = RequestMock.do_request_patch(
            template_version_manager_views.UserTemplateVersionManagerOrdering.as_view(),
            mock_user,
            data={"template_list": []},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



# Generated by Django 3.2.18 on 2023-05-25 19:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0004_template_ordering"),
    ]

    operations = [
        migrations.AlterField(
            model_name="template",
            name="dependencies",
            field=models.ManyToManyField(
                blank=True, default=[], to="core_main_app.Template"
            ),
        ),
    ]



""" Authentication tests for views
"""
from django.contrib.auth.models import AnonymousUser
from rest_framework import status

from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    IntegrationTransactionTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_main_app.views.user import views as user_views


class TestSetTimezone(IntegrationTransactionTestCase):
    """TestSetTimezone"""

    def setUp(self):
        """setUp

        Returns:

        """
        self.user1 = create_mock_user(user_id="1")
        self.anonymous = AnonymousUser()

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            user_views.set_timezone,
            self.anonymous,
        )

        self.assertEqual(response.status_code, status.HTTP_302_FOUND)

    def test_authenticated_returns_http_200(self):
        """test_authenticated_returns_http_403

        Returns:

        """

        response = RequestMock.do_request_get(
            user_views.set_timezone,
            self.user1,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



"""Utils for mongodb raw query
"""
from core_main_app.utils.raw_query.common import (
    check_user_filter,
    check_workspace_filter,
)


def add_access_criteria(
    query, accessible_workspaces, user, workspace_filter=None, user_filter=None
):
    """Add access criteria to the query.

    Args:
        query:
        accessible_workspaces:
        user:
        workspace_filter:
        user_filter:

    Returns:

    """
    # check if user can filter by user
    check_user_filter(user_filter, user)
    # check if user can filter by workspace
    check_workspace_filter(workspace_filter, accessible_workspaces)
    # get access criteria query
    access_criteria = _get_accessible_criteria(accessible_workspaces, user)
    # add access criteria to original query
    query = {"$and": [query, access_criteria]}
    # return query
    return query


def add_aggregate_access_criteria(query, accessible_workspaces, user):
    """Add access criteria to an aggregation query.

    Args:
        query:
        accessible_workspaces:
        user:

    Returns:

    """
    has_match_criteria = False
    access_criteria = _get_accessible_criteria(accessible_workspaces, user)
    # Update $match with access criteria if exist.
    for elt in query:
        if "$match" in elt:
            elt["$match"].update(access_criteria)
            has_match_criteria = True
            break
    # Add $match with access criteria if does not exist.
    if not has_match_criteria:
        query.insert(0, {"$match": access_criteria})

    return query


def _get_accessible_criteria(accessible_workspaces, user):
    """Get accessible criteria.

    Args:
        accessible_workspaces:
        user:

    Returns:

    """
    # workspace should be in list of accessible workspaces
    workspace_criteria = {"workspace": {"$in": accessible_workspaces}}
    # user_id should have the id of the user making the query
    user_criteria = {"user_id": user.id}
    # access granted if workspace or user criteria true
    access_criteria = {"$or": [workspace_criteria, user_criteria]}
    # return access criteria
    return access_criteria



""" Unit tests of system API
"""
from collections import OrderedDict
from unittest.case import TestCase
from unittest.mock import patch

from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.components.workspace.models import Workspace
from core_main_app.system import api as system_api


def _get_template():
    """get template

    Returns:

    """

    template = Template()
    template.pk = 1
    xsd = (
        "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'>"
        "<xs:element name='tag'></xs:element></xs:schema>"
    )
    template.content = xsd
    return template


def _get_workspace():
    """get workspace

    Returns:

    """
    workspace = Workspace()
    workspace.title = "title"
    workspace.owner = 1
    workspace.is_public = True
    workspace.pk = 1
    return workspace


class TestSystemGetAllExcept(TestCase):
    """TestSystemGetAllExcept"""

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_return_data_object(
        self, mock_get_all_except
    ):
        """test system get all except return data object

        Args:
            mock_get_all_except:

        Returns:

        """
        mock_data = Data(
            template=_get_template(),
            user_id="1",
            dict_content=OrderedDict(),
            title="title",
        )
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except(["1"])
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_return_correct_count(
        self, mock_get_all_except
    ):
        """test system get all except return correct count

        Args:
            mock_get_all_except:

        Returns:

        """
        mock_data = Data(
            template=_get_template(),
            user_id="1",
            dict_content=OrderedDict(),
            title="title",
        )
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except(["1"])
        self.assertEqual(len(result), 1)

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_empty_list_return_data_object(
        self, mock_get_all_except
    ):
        """test system get all except empty list return data object

        Args:
            mock_get_all_except:

        Returns:

        """
        mock_data = Data(
            template=_get_template(),
            user_id="1",
            dict_content=OrderedDict(),
            title="title",
        )
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except([])
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_empty_list_return_correct_count(
        self, mock_get_all_except
    ):
        """test system get all except empty list return correct count

        Args:
            mock_get_all_except:

        Returns:

        """
        mock_data = Data(
            template=_get_template(),
            user_id="1",
            dict_content=OrderedDict(),
            title="title",
        )
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except([])
        self.assertEqual(len(result), 1)

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_bad_id_return_data_object(
        self, mock_get_all_except
    ):
        """test system get all except bad id return data object

        Args:
            mock_get_all_except:

        Returns:

        """
        mock_data = Data(
            template=_get_template(),
            user_id="1",
            dict_content=OrderedDict(),
            title="title",
        )
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except(["1"])
        self.assertTrue(all(isinstance(item, Data) for item in result))

    @patch.object(Data, "get_all_except")
    def test_system_get_all_except_bad_id_return_correct_count(
        self, mock_get_all_except
    ):
        """test system get all except bad id return correct count

        Args:
            mock_get_all_except:

        Returns:

        """
        mock_data = Data(
            template=_get_template(),
            user_id="1",
            dict_content=OrderedDict(),
            title="title",
        )
        mock_get_all_except.return_value = [mock_data]

        result = system_api.get_all_except(["1"])
        self.assertEqual(len(result), 1)



"""
Template models
"""
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.validators import RegexValidator
from django.db import models, IntegrityError
from django.db.models import Q

from core_main_app.commons import exceptions
from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.version_manager.models import Version
from core_main_app.settings import XSD_UPLOAD_DIR, CHECKSUM_ALGORITHM
from core_main_app.utils.checksum import compute_checksum
from core_main_app.utils.storage.storage import core_file_storage
from core_main_app.utils.validation.regex_validation import (
    not_empty_or_whitespaces,
)


class Template(Version):
    """Represents an XML schema template that defines the structure of data"""

    class_name = "Template"

    version_manager = models.ForeignKey(
        TemplateVersionManager,
        on_delete=models.CASCADE,
        null=True,
        default=None,
    )
    filename = models.CharField(
        validators=[
            RegexValidator(
                regex=NOT_EMPTY_OR_WHITESPACES,
                message="Title must not be empty or only whitespaces",
                code="invalid_title",
            ),
        ],
        max_length=200,
    )
    file = models.FileField(
        blank=False,
        max_length=250,
        # NOTE: needed to check owner during upload (cf. core_main_app.utils.xml._get_schema_location_uri)
        upload_to=XSD_UPLOAD_DIR,
        storage=core_file_storage(model="template"),
    )
    checksum = models.CharField(
        max_length=512, blank=True, default=None, null=True
    )
    user = models.CharField(
        blank=True, max_length=200, null=True, default=None
    )
    hash = models.CharField(max_length=200)
    _display_name = models.CharField(blank=True, max_length=200)
    dependencies = models.ManyToManyField(
        "self", blank=True, default=[], symmetrical=False
    )
    creation_date = models.DateTimeField(auto_now_add=True)
    _cls = models.CharField(default="Template", max_length=200)
    _content = None

    @property
    def content(self):
        """Read template content

        Returns:

        """
        if not self._content:
            self._content = self.file.read().decode("utf-8")
        return self._content

    @content.setter
    def content(self, xsd_content):
        """Set template content

        Args:
            xsd_content:

        Returns:

        """
        # Set template content
        self._content = xsd_content

    @staticmethod
    def get_all(is_cls, users=None):
        """Return all templates.

        Args:
            is_cls (bool): True if filtering by templates only.
            users (list|None): List of users having access to the template.

        Returns:
            list<Template> - List of template following the query parameters.
        """
        template_query = Q()

        if is_cls:  # will return all Template object only
            template_query &= Q(_cls=Template.class_name)

        if users is not None:  # select specific user if it is defined
            template_query &= users

        return Template.objects.filter(template_query).all()

    @staticmethod
    def get_by_id(template_id):
        """Return a template by its id.

        Args:
            template_id:

        Returns:

        """
        try:
            return Template.objects.get(pk=template_id)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_all_by_hash(template_hash, users):
        """Return all template having the given hash.

        Args:
            template_hash: Template hash.
            users:

        Returns:
            List of Template instance.

        """
        if users is not None:
            return Template.objects.filter(Q(hash=template_hash) & users).all()
        return Template.objects.filter(hash=template_hash).all()

    @staticmethod
    def get_all_by_hash_list(template_hash_list, users):
        """Return all template having the given hash list.

        Args:
            template_hash_list: Template hash list.
            users:

        Returns:
            List of Template instance.

        """
        if users is not None:
            return Template.objects.filter(
                Q(hash__in=template_hash_list) & users
            ).all()
        return Template.objects.filter(hash__in=template_hash_list).all()

    @staticmethod
    def get_all_by_id_list(template_id_list, users=None):
        """Return all template with id in list.

        Args:
            template_id_list:
            users:

        Returns:

        """
        if users is not None:
            return Template.objects.filter(
                Q(pk__in=template_id_list) & users
            ).all()
        return Template.objects.filter(pk__in=template_id_list).all()

    @property
    def display_name(self):
        """Return template name to display.

        Returns:

        """
        if self._display_name is not None:
            return self._display_name
        return self.filename

    @display_name.setter
    def display_name(self, value):
        """Set template name to display.

        Args:
            value:

        Returns:

        """
        self._display_name = value

    def save_template(self):
        """Custom save.

        Returns:
            Saved Instance.

        """
        try:
            self._cls = self.class_name
            if self._content:
                self.file = SimpleUploadedFile(
                    name=self.filename,
                    content=self._content.encode("utf-8"),
                    content_type="application/xml",
                )
            not_empty_or_whitespaces(self.filename)
            if self.content and CHECKSUM_ALGORITHM:
                self.checksum = compute_checksum(
                    self.content.encode(), CHECKSUM_ALGORITHM
                )
            self.save()
        except IntegrityError as exception:
            raise exceptions.NotUniqueError(str(exception))
        except ValidationError as exception:
            raise exception
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    def __str__(self):
        """Template object as string

        Returns:

        """
        return self.display_name



""" Migration to order Blob object by creation date.

Generated by Django 4.2.18 on 2023-08-21 13:14
"""
from django.db import migrations


class Migration(migrations.Migration):
    """Migration class."""

    dependencies = [
        ("core_main_app", "0007_user_preferences"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="blob",
            options={"ordering": ["-creation_date"]},
        ),
    ]



"""Pagination configuration for rest_framework
"""
from rest_framework.pagination import PageNumberPagination

from core_main_app.settings import RESULTS_PER_PAGE
from core_main_app.utils.pagination.mongoengine_paginator.paginator import (
    MongoenginePaginator,
)
from django.conf import settings


class StandardResultsSetPagination(PageNumberPagination):
    """Standard Results Set Pagination"""

    page_size = RESULTS_PER_PAGE
    if settings.MONGODB_INDEXING:
        django_paginator_class = MongoenginePaginator



""" Abstract Data model
"""

from django.conf import settings
from django.contrib.postgres.search import SearchVectorField
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.validators import RegexValidator
from django.db import models, IntegrityError

from core_main_app.commons import exceptions
from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES
from core_main_app.settings import (
    SEARCHABLE_DATA_OCCURRENCES_LIMIT,
    XML_POST_PROCESSOR,
    XML_FORCE_LIST,
    CHECKSUM_ALGORITHM,
)
from core_main_app.utils import xml as xml_utils
from core_main_app.utils.checksum import compute_checksum
from core_main_app.utils.datetime import datetime_now
from core_main_app.utils.storage.storage import (
    core_file_storage,
    user_directory_path,
)


class AbstractData(models.Model):
    """AbstractData object"""

    dict_content = models.JSONField(blank=True, null=True)
    title = models.CharField(
        blank=False,
        validators=[
            RegexValidator(
                regex=NOT_EMPTY_OR_WHITESPACES,
                message="Title must not be empty or only whitespaces",
                code="invalid_title",
            ),
        ],
        max_length=200,
    )
    xml_file = models.FileField(
        blank=False,
        max_length=250,
        upload_to=user_directory_path,
        storage=core_file_storage(model="data"),
    )
    checksum = models.CharField(
        max_length=512, blank=True, default=None, null=True
    )
    vector_column = SearchVectorField(null=True)
    creation_date = models.DateTimeField(blank=True, default=None, null=True)
    last_modification_date = models.DateTimeField(
        blank=True, default=None, null=True
    )
    last_change_date = models.DateTimeField(
        blank=True, default=None, null=True
    )
    _xml_content = None

    class Meta:
        """Meta"""

        abstract = True

    @property
    def xml_content(self):
        """Get xml content - read from a saved file.

        Returns:

        """
        # private field xml_content not set yet, and reference to xml_file to read is set
        if self._xml_content is None and self.xml_file.name:
            # read xml file into xml_content field
            xml_content = self.xml_file.read()
            try:
                self._xml_content = (
                    xml_content.decode("utf-8") if xml_content else xml_content
                )
            except AttributeError:
                self._xml_content = xml_content
        # return xml content
        return self._xml_content

    @xml_content.setter
    def xml_content(self, value):
        """Set xml content - to be saved as a file.

        Args:
            value:

        Returns:

        """
        # update modification times
        self.last_modification_date = datetime_now()
        # update content
        self._xml_content = value

    def get_dict_content(self):
        """Get dict_content from object or from MongoDB

        Returns:

        """
        raise NotImplementedError("get_dict_content is not implemented")

    def convert_and_save(self):
        """Save Data object and convert the xml to dict if needed.

        Returns:

        """
        self.convert_to_dict()
        self.convert_to_file()

        self.save_object()

    def convert_to_dict(self):
        """Convert the xml contained in xml_content into a dictionary.

        Returns:

        """
        # if data stored in mongo, don't store dict_content
        if settings.MONGODB_INDEXING:
            return
        # transform xml content into a dictionary
        self.dict_content = xml_utils.raw_xml_to_dict(
            self.xml_content,
            postprocessor=XML_POST_PROCESSOR,
            force_list=XML_FORCE_LIST,
            list_limit=SEARCHABLE_DATA_OCCURRENCES_LIMIT,
        )

    def convert_to_file(self):
        """Convert the xml string into a file.

        Returns:

        """
        try:
            xml_content = self.xml_content.encode("utf-8")
        except UnicodeEncodeError:
            xml_content = self.xml_content

        self.xml_file = SimpleUploadedFile(
            name=self.title,
            content=xml_content,
            content_type="application/xml",
        )

    def save_object(self):
        """Custom save. Set the datetime fields and save.

        Returns:

        """
        try:
            # initialize times
            now = datetime_now()
            # update change date every time the data is updated
            self.last_change_date = now
            if not self.id:
                # initialize when first created
                self.creation_date = now
                # initialize when first saved, then only updates when content is updated
                self.last_modification_date = now
            if self.xml_content and CHECKSUM_ALGORITHM:
                self.checksum = compute_checksum(
                    self.xml_content.encode(), CHECKSUM_ALGORITHM
                )
            self.save()
        except IntegrityError as exception:
            raise exceptions.NotUniqueError(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))



""" BLOB API
"""
from core_main_app.access_control.api import (
    can_change_owner,
    can_write,
)
from core_main_app.access_control.api import (
    has_perm_administration,
    can_read_or_write_in_workspace,
    can_read_id,
)
from core_main_app.access_control.decorators import access_control
from core_main_app.commons import exceptions
from core_main_app.components.blob.access_control import (
    can_write_blob_workspace,
    can_write_blob,
    can_write_metadata,
    can_write_metadata_list,
)
from core_main_app.components.blob.models import Blob


@access_control(can_write_blob)
def insert(blob, user):
    """Insert the blob in the blob repository.

    Args:
        blob:
        user:

    Returns:

    """
    # if blob is already created
    if blob.id:
        raise exceptions.ApiError(
            "Unable to save the blob: change is not allowed. Insert method only for blob creation."
        )
    # if blob is not set
    if blob.blob is None:
        raise exceptions.ApiError(
            "Unable to save the blob: blob field is not set."
        )
    # save blob in database
    blob.save_object()
    return blob


@access_control(can_write_blob_workspace)
def assign(blob, workspace, user):
    """Assign blob to a workspace.

    Args:
        blob:
        workspace:
        user:

    Returns:

    """
    blob.workspace = workspace
    return blob.save()


@access_control(can_write)
def delete(blob, user):
    """Delete the blob.

    Args:
        blob:
        user:

    Returns:

    """
    # delete blob in database
    return blob.delete()


@access_control(can_read_id)
def get_by_id(blob_id, user):
    """Return blob by its id.

    Args:
        blob_id:
        user:

    Returns:

    """
    return Blob.get_by_id(blob_id)


@access_control(has_perm_administration)
def get_all(user):
    """Return all blobs.

    Args:

    Returns:
        List of Blob instances.

    """
    return Blob.get_all()


def get_all_by_user(user):
    """Return all blobs by user.

    Args:
        user: User

    Returns:
        List of Blob instances for the given user id.

    """
    return Blob.get_all_by_user_id(str(user.id))


@access_control(can_read_or_write_in_workspace)
def get_all_by_workspace(workspace, user):
    """Get all data that belong to the workspace.

    Args:
        workspace:
        user:

    Returns:

    """
    return Blob.get_all_by_workspace(workspace)


@access_control(can_change_owner)
def change_owner(blob, new_user, user):
    """Change blob's owner.

    Args:
        blob:
        user:
        new_user:

    Returns:
    """
    # FIXME: user can transfer data to anybody, too permissive
    blob.user_id = str(new_user.id)
    blob.save()


def get_none():
    """Returns None object, used by blobs

    Returns:

    """
    return Blob.get_none()


@access_control(can_write_metadata)
def add_metadata(blob, metadata, user):
    """Add metadata to blob

    Args:
        blob:
        metadata:
        user:

    Returns:

    """
    blob._metadata.add(metadata)
    blob.save()


@access_control(can_write_metadata_list)
def add_metadata_list(blob, metadata_list, user):
    """Add list of metadata to blob

    Args:
        blob:
        metadata_list:
        user:

    Returns:

    """
    for metadata in metadata_list:
        blob._metadata.add(metadata)
    blob.save()


@access_control(can_write_metadata)
def remove_metadata(blob, metadata, user):
    """Remove metadata from blob

    Args:
        blob:
        metadata:
        user:

    Returns:

    """
    blob._metadata.remove(metadata)
    blob.save()
    metadata._blob = None
    metadata.save()



""" Urls test class
"""
from unittest import TestCase

from django.test import override_settings

from core_main_app.utils.urls import get_auth_urls


class TestUrls(TestCase):
    """TestUrls"""

    @override_settings(ENABLE_SAML2_SSO_AUTH=True)
    def test_urls_with_ENABLE_SAML2_SSO_AUTH(self):
        """test_urls_with_ENABLE_SAML2_SSO_AUTH

        Returns:

        """
        urlpatterns = get_auth_urls()
        self.assertTrue("core_main_app_login" in str(urlpatterns))
        self.assertTrue("saml2" in str(urlpatterns))

    @override_settings(ENABLE_SAML2_SSO_AUTH=False)
    def test_urls(self):
        """test_urls

        Returns:

        """
        urlpatterns = get_auth_urls()
        self.assertTrue("core_main_app_login" in str(urlpatterns))



""" Unit tests for `core_main_app.components.data.access_control`.
"""
from unittest import TestCase
from unittest.mock import patch

from unittest.mock import MagicMock

from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.components.data import access_control as data_acl


class TestCanReadAggregateQuery(TestCase):
    """Unit tests for `can_read_aggregate_query` function."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_kwargs = {
            "func": MagicMock(),
            "query": MagicMock(),
            "user": create_mock_user("1"),
        }

    @patch.object(data_acl, "_update_can_read_aggregate_query")
    @patch.object(data_acl, "check_anonymous_access")
    def test_superuser_returns_func_with_args(
        self, mock_check_anonymous_access, mock_update_can_read_aggregate_query
    ):
        """test_superuser_returns_func_with_args"""
        self.mock_kwargs["user"] = create_mock_user("1", is_superuser=True)

        self.assertEqual(
            data_acl.can_read_aggregate_query(**self.mock_kwargs),
            self.mock_kwargs["func"](
                self.mock_kwargs["query"], self.mock_kwargs["user"]
            ),
        )

    @patch.object(data_acl, "_update_can_read_aggregate_query")
    @patch.object(data_acl, "check_anonymous_access")
    def test_check_anonymous_access_called(
        self, mock_check_anonymous_access, mock_update_can_read_aggregate_query
    ):
        """test_check_anonymous_access_called"""

        data_acl.can_read_aggregate_query(**self.mock_kwargs)
        mock_check_anonymous_access.assert_called_with(
            self.mock_kwargs["user"]
        )

    @patch.object(data_acl, "_update_can_read_aggregate_query")
    @patch.object(data_acl, "check_anonymous_access")
    def test_update_can_read_aggregate_query_called(
        self, mock_check_anonymous_access, mock_update_can_read_aggregate_query
    ):
        """test_update_can_read_aggregate_query_called"""

        data_acl.can_read_aggregate_query(**self.mock_kwargs)
        mock_update_can_read_aggregate_query.assert_called_with(
            self.mock_kwargs["query"], self.mock_kwargs["user"]
        )

    @patch.object(data_acl, "_update_can_read_aggregate_query")
    @patch.object(data_acl, "check_anonymous_access")
    def test_returns_func_with_args(
        self, mock_check_anonymous_access, mock_update_can_read_aggregate_query
    ):
        """test_returns_func_with_args"""

        self.assertEqual(
            data_acl.can_read_aggregate_query(**self.mock_kwargs),
            self.mock_kwargs["func"](
                self.mock_kwargs["query"], self.mock_kwargs["user"]
            ),
        )



""" Unit tests for Lock components
"""
from unittest import TestCase

from unittest.mock import Mock

from core_main_app.commons.exceptions import LockError
from core_main_app.components.lock.api import _check_object_locked
from core_main_app.settings import LOCK_OBJECT_TTL
from core_main_app.utils.datetime import datetime_now, datetime_timedelta
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestCheckObjectLocked(TestCase):
    """Test _check_object_locked function"""

    def setUp(self) -> None:
        self.mock_object = Mock()
        self.mock_user = create_mock_user(user_id=1)
        self.mock_lock = Mock()

    def test_get_object_locked_fails_returns_false(self):
        """test_get_object_locked_fails_returns_false"""
        self.mock_lock.get_object_locked.side_effect = Exception(
            "mock_get_object_locked_exception"
        )

        result = _check_object_locked(
            self.mock_object, self.mock_user, self.mock_lock
        )
        self.assertFalse(result)

    def test_object_lock_expired_returns_false(self):
        """test_object_lock_expired_returns_false"""
        mock_lock_object = Mock()
        mock_lock_object.lock_date = datetime_now() - datetime_timedelta(
            seconds=10 * LOCK_OBJECT_TTL
        )
        self.mock_lock.get_object_locked.return_value = mock_lock_object

        result = _check_object_locked(
            self.mock_object, self.mock_user, self.mock_lock
        )
        self.assertFalse(result)

    def test_user_is_object_owner_returns_true(self):
        """test_user_is_object_owner_returns_true"""
        mock_lock_object = Mock()
        mock_lock_object.lock_date = datetime_now()
        mock_lock_object.user_id = str(self.mock_user.id)
        self.mock_lock.get_object_locked.return_value = mock_lock_object

        result = _check_object_locked(
            self.mock_object, self.mock_user, self.mock_lock
        )
        self.assertTrue(result)

    def test_user_not_owner_raises_error(self):
        """test_user_not_owner_raises_error"""
        mock_lock_object = Mock()
        mock_lock_object.lock_date = datetime_now()
        mock_lock_object.user_id = "2"
        self.mock_lock.get_object_locked.return_value = mock_lock_object

        with self.assertRaises(LockError):
            _check_object_locked(
                self.mock_object, self.mock_user, self.mock_lock
            )



""" Fixtures files for User Preferences
"""

from core_main_app.components.user_preferences.models import UserPreferences

from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)


class UserPreferencesFixtures(FixtureInterface):
    """User Preferences Fixture"""

    user_preferences_1 = None
    user_preferences_2 = None
    user_preferences_3 = None
    users_preferences_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_users_preferences_collection()

    def generate_users_preferences_collection(self):
        """Generate a Data collection.

        Returns:

        """
        self.user_preferences_1 = UserPreferences(user_id="1", timezone="UTC")
        self.user_preferences_1.save()
        self.user_preferences_2 = UserPreferences(
            user_id="2", timezone="US/Central"
        )
        self.user_preferences_2.save()
        self.user_preferences_3 = UserPreferences(user_id="3", timezone="UTC")
        self.user_preferences_3.save()

        self.users_preferences_collection = [
            self.user_preferences_1,
            self.user_preferences_2,
            self.user_preferences_3,
        ]



""" Parser utils test class
"""
from unittest import TestCase

from core_main_app.utils.parser import get_parser
from core_parser_app.tools.parser.parser import XSDParser


class TestGetParser(TestCase):
    """Test to_bool"""

    def test_get_parser_returns_parser(self):
        """test_get_parser_returns_parser

        Returns:

        """
        self.assertIsInstance(get_parser(), XSDParser)



""" Url router for the administration site
"""
from django.contrib import admin
from django.contrib.admin import AdminSite
from django.contrib.admin.views.decorators import staff_member_required
from django.urls import re_path
from django.urls import reverse_lazy
from django.views.generic.base import RedirectView

from core_main_app.commons.enums import WEB_PAGE_TYPES
from core_main_app.components.blob.admin_site import CustomBlobAdmin
from core_main_app.components.blob.models import Blob
from core_main_app.components.data.admin_site import CustomDataAdmin
from core_main_app.components.data.models import Data
from core_main_app.components.lock.admin_site import CustomDatabaseLockAdmin
from core_main_app.components.lock.models import DatabaseLockObject
from core_main_app.components.template.admin_site import CustomTemplateAdmin
from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager.admin_site import (
    CustomTemplateVersionManagerAdmin,
)
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.components.template_xsl_rendering.admin_site import (
    CustomTemplateXslRenderingAdmin,
)
from core_main_app.components.template_xsl_rendering.models import (
    TemplateXslRendering,
)
from core_main_app.components.user_preferences.admin_site import (
    CustomUserPreferencesAdmin,
)
from core_main_app.components.user_preferences.models import UserPreferences
from core_main_app.components.web_page_login import api as login_page_api
from core_main_app.components.workspace.admin_site import CustomWorkspaceAdmin
from core_main_app.components.workspace.models import Workspace
from core_main_app.components.xsl_transformation.admin_site import (
    CustomXslTransformationAdmin,
)
from core_main_app.components.xsl_transformation.models import (
    XslTransformation,
)
from core_main_app.utils.rendering import admin_render
from core_main_app.views.admin import views as admin_views, ajax as admin_ajax
from core_main_app.views.admin.views import WebPageView
from core_main_app.views.common import views as common_views

admin_urls = [
    re_path(
        r"^login/message/$",
        staff_member_required(
            WebPageView.as_view(
                api=login_page_api,
                get_redirect="core_main_app/admin/web_page/login_page.html",
                post_redirect="core-admin:core_main_app_login_page",
                web_page_type=WEB_PAGE_TYPES["login"],
            )
        ),
        name="core_main_app_login_page",
    ),
    re_path(
        r"^login",
        RedirectView.as_view(url=reverse_lazy("core_main_app_login")),
    ),
    re_path(
        r"^logout",
        RedirectView.as_view(url=reverse_lazy("core_main_app_logout")),
    ),
    re_path(
        r"^data$",
        staff_member_required(
            common_views.ViewData.as_view(
                administration=True,
                template="core_main_app/admin/data/detail.html",
            )
        ),
        name="core_main_app_data_detail",
    ),
    re_path(
        r"^templates$",
        admin_views.manage_templates,
        name="core_main_app_templates",
    ),
    re_path(
        r"^template/upload$",
        admin_views.upload_template,
        name="core_main_app_upload_template",
    ),
    re_path(
        r"^template/data-migration",
        admin_views.data_migration,
        name="core_main_app_data_migration",
    ),
    re_path(
        r"^template/upload/(?P<version_manager_id>\w+)",
        admin_views.upload_template_version,
        name="core_main_app_upload_template_version",
    ),
    re_path(
        r"^template/versions/(?P<version_manager_id>\w+)",
        admin_views.manage_template_versions,
        name="core_main_app_manage_template_versions",
    ),
    re_path(
        r"^template/xslt/(?P<template_id>\w+)",
        staff_member_required(
            common_views.TemplateXSLRenderingView.as_view(
                rendering=admin_render,
                template_name="core_main_app/admin/templates_xslt/main.html",
                save_redirect="core-admin:core_main_app_manage_template_versions",
                back_to_url="core-admin:core_main_app_manage_template_versions",
            )
        ),
        name="core_main_app_template_xslt",
    ),
    re_path(
        r"^dashboard$", admin_views.admin_home, name="core_main_app_admin_home"
    ),
    re_path(
        r"^template/resolve-dependencies",
        admin_ajax.resolve_dependencies,
        name="core_main_app_resolve_dependencies",
    ),
    re_path(
        r"^xslt/(?P<pk>[\w-]+)/edit/$",
        staff_member_required(admin_ajax.EditXSLTView.as_view()),
        name="core_main_app_edit_xslt",
    ),
    re_path(
        r"^xslt$",
        staff_member_required(admin_views.XSLTView.as_view()),
        name="core_main_app_xslt",
    ),
    re_path(
        r"^xslt/upload$",
        staff_member_required(admin_views.UploadXSLTView.as_view()),
        name="core_main_app_upload_xslt",
    ),
    re_path(
        r"^xslt/(?P<pk>[\w-]+)/delete/$",
        staff_member_required(admin_ajax.DeleteXSLTView.as_view()),
        name="core_main_app_delete_xslt",
    ),
    re_path(
        r"^edit-rights/(?P<workspace_id>\w+)$",
        staff_member_required(
            common_views.EditWorkspaceRights.as_view(
                administration=True,
                template="core_main_app/admin/workspaces/edit_rights.html",
            )
        ),
        name="core_main_edit_rights_workspace",
    ),
]


admin.site.register(Data, CustomDataAdmin)
admin.site.register(Blob, CustomBlobAdmin)
admin.site.register(Workspace, CustomWorkspaceAdmin)
admin.site.register(TemplateVersionManager, CustomTemplateVersionManagerAdmin)
admin.site.register(Template, CustomTemplateAdmin)
admin.site.register(XslTransformation, CustomXslTransformationAdmin)
admin.site.register(TemplateXslRendering, CustomTemplateXslRenderingAdmin)
admin.site.register(DatabaseLockObject, CustomDatabaseLockAdmin)
admin.site.register(UserPreferences, CustomUserPreferencesAdmin)


class CoreAdminSite(AdminSite):
    """Admin site for Core apps"""

    def get_urls(self):
        return admin_urls


core_admin_site = CoreAdminSite(name="core-admin")
